@@ -28,6 +28,7 @@
 #include <fcntl.h>
 #include <gtk/gtkmain.h>
 #include <gtk/gtksignal.h>
+#include <libgnomeui/gnome-thumbnail.h>
 #include <libgnomevfs/gnome-vfs-async-ops.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
@@ -37,6 +38,10 @@
 #include "gthumb-marshal.h"
 #include "file-utils.h"
 #include "glib-utils.h"
+#include "gconf-utils.h"
+#include "gth-exif-utils.h"
+#include "pixbuf-utils.h"
+#include "preferences.h"
 
 
 #define REFRESH_RATE 5
@@ -44,49 +49,52 @@ G_LOCK_DEFINE_STATIC (pixbuf_loader_lock);
 
 
 struct _ImageLoaderPrivateData {
-	GdkPixbuf            *pixbuf;
-	GdkPixbufAnimation   *animation;
+	GdkPixbuf             *pixbuf;
+	GdkPixbufAnimation    *animation;
 
-	gboolean              as_animation; /* Whether to load the image in a
+	GnomeThumbnailFactory *thumb_factory;
+
+	gboolean               as_animation; /* Whether to load the image in a
 					     * GdkPixbufAnimation structure. */
 
-	GnomeVFSURI          *uri;
-	
-	GnomeVFSAsyncHandle  *info_handle;
+	GnomeVFSURI           *uri;
+	const char            *mime_type;
 
-	GnomeVFSFileSize      bytes_read;
-	GnomeVFSFileSize      bytes_total;
+	GnomeVFSAsyncHandle   *info_handle;
 
-	gboolean              done;
-	gboolean              error;
-	gboolean              loader_done;
-	gboolean              interrupted;
-	gboolean              loading;
+	GnomeVFSFileSize       bytes_read;
+	GnomeVFSFileSize       bytes_total;
 
-	GTimer               *timer;
-	int                   priority;
+	gboolean               done;
+	gboolean               error;
+	gboolean               loader_done;
+	gboolean               interrupted;
+	gboolean               loading;
 
-	DoneFunc              done_func;
-	gpointer              done_func_data;
+	GTimer                *timer;
+	int                    priority;
 
-	gboolean              emit_signal;
+	DoneFunc               done_func;
+	gpointer               done_func_data;
 
-	guint                 check_id;
-	guint                 idle_id;
+	gboolean               emit_signal;
 
-	GThread              *thread;
+	guint                  check_id;
+	guint                  idle_id;
 
-	GMutex               *yes_or_no;
+	GThread               *thread;
 
-	gboolean              exit_thread;
-	GMutex               *exit_thread_mutex;
+	GMutex                *yes_or_no;
 
-	gboolean              start_loading;
-	GMutex               *start_loading_mutex;
-	GCond                *start_loading_cond;
+	gboolean               exit_thread;
+	GMutex                *exit_thread_mutex;
 
-	LoaderFunc            loader;
-	gpointer              loader_data;
+	gboolean               start_loading;
+	GMutex                *start_loading_mutex;
+	GCond                 *start_loading_cond;
+
+	LoaderFunc             loader;
+	gpointer               loader_data;
 };
 
 
@@ -114,7 +122,7 @@ image_loader_finalize__step2 (GObject *object)
 	g_mutex_lock (priv->yes_or_no);
 	if (priv->pixbuf != NULL)
 		g_object_unref (G_OBJECT (priv->pixbuf));
-	
+
 	if (priv->animation != NULL)
 		g_object_unref (G_OBJECT (priv->animation));
 
@@ -157,7 +165,7 @@ static void image_loader_stop_common (ImageLoader *il,
 				      gboolean     use_idle_cb);
 
 
-static void 
+static void
 image_loader_finalize (GObject *object)
 {
 	ImageLoader            *il;
@@ -165,17 +173,20 @@ image_loader_finalize (GObject *object)
 
         g_return_if_fail (object != NULL);
         g_return_if_fail (IS_IMAGE_LOADER (object));
-  
+
         il = IMAGE_LOADER (object);
 	priv = il->priv;
 
+	if (priv->thumb_factory != NULL)
+		g_object_unref (priv->thumb_factory);
+
 	if (priv->idle_id != 0) {
 		g_source_remove (priv->idle_id);
 		priv->idle_id = 0;
 	}
-	
-	image_loader_stop_common (il, 
-				  (DoneFunc) image_loader_finalize__step2, 
+
+	image_loader_stop_common (il,
+				  (DoneFunc) image_loader_finalize__step2,
 				  object,
 				  FALSE,
 				  FALSE);
@@ -195,8 +206,8 @@ image_loader_class_init (ImageLoaderClass *class)
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (ImageLoaderClass, image_error),
 			      NULL, NULL,
-			      gthumb_marshal_VOID__VOID,
-			      G_TYPE_NONE, 
+			      g_cclosure_marshal_VOID__VOID,
+			      G_TYPE_NONE,
 			      0);
 	image_loader_signals[IMAGE_DONE] =
 		g_signal_new ("image_done",
@@ -204,8 +215,8 @@ image_loader_class_init (ImageLoaderClass *class)
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (ImageLoaderClass, image_done),
 			      NULL, NULL,
-			      gthumb_marshal_VOID__VOID,
-			      G_TYPE_NONE, 
+			      g_cclosure_marshal_VOID__VOID,
+			      G_TYPE_NONE,
 			      0);
 	image_loader_signals[IMAGE_PROGRESS] =
 		g_signal_new ("image_progress",
@@ -214,7 +225,7 @@ image_loader_class_init (ImageLoaderClass *class)
 			      G_STRUCT_OFFSET (ImageLoaderClass, image_progress),
 			      NULL, NULL,
 			      gthumb_marshal_VOID__FLOAT,
-			      G_TYPE_NONE, 
+			      G_TYPE_NONE,
 			      1, G_TYPE_FLOAT);
 
 	object_class->finalize = image_loader_finalize;
@@ -234,6 +245,7 @@ image_loader_init (ImageLoader *il)
 	il->priv = g_new0 (ImageLoaderPrivateData, 1);
 	priv = il->priv;
 
+	priv->thumb_factory = NULL;
 	priv->pixbuf = NULL;
 	priv->animation = NULL;
 	priv->uri = NULL;
@@ -269,7 +281,22 @@ image_loader_init (ImageLoader *il)
 	priv->loader = NULL;
 	priv->loader_data = NULL;
 
-	priv->thread = g_thread_create (load_image_thread, il, TRUE, NULL);
+	/*priv->thread = g_thread_create (load_image_thread, il, TRUE, NULL);*/
+
+	/* The g_thread_create function assigns a very large default stacksize for each
+	   thread (10 MB on FC6), which is probably excessive. 16k seems to be
+	   sufficient. To be conversative, we'll try 32k. Use g_thread_create_full to
+	   manually specify a small stack size. See Bug 310749 - Memory usage.
+	   This reduces the virtual memory requirements, and the "writeable/private"
+	   figure reported by "pmap -d". */
+
+	priv->thread = g_thread_create_full (load_image_thread,
+					     il,
+					     32768,
+					     TRUE,
+					     TRUE,
+					     G_THREAD_PRIORITY_NORMAL,
+					     NULL);
 }
 
 
@@ -301,7 +328,7 @@ image_loader_get_type ()
 }
 
 
-GObject*     
+GObject*
 image_loader_new (const gchar *path,
 		  gboolean     as_animation)
 {
@@ -312,7 +339,9 @@ image_loader_new (const gchar *path,
 	priv = (ImageLoaderPrivateData*) il->priv;
 
 	priv->as_animation = as_animation;
-	image_loader_set_path (il, path);
+	image_loader_set_path (il, path, NULL);
+
+	priv->thumb_factory = gnome_thumbnail_factory_new (GNOME_THUMBNAIL_SIZE_LARGE);
 
 	return G_OBJECT (il);
 }
@@ -334,7 +363,8 @@ image_loader_set_loader (ImageLoader *il,
 
 void
 image_loader_set_path (ImageLoader *il,
-		       const char  *path)
+		       const char  *path,
+		       const char  *mime_type)
 {
 	ImageLoaderPrivateData *priv;
 
@@ -344,11 +374,15 @@ image_loader_set_path (ImageLoader *il,
 
 	g_mutex_lock (priv->yes_or_no);
 
+	if (mime_type == NULL)
+		mime_type = get_file_mime_type (path, eel_gconf_get_boolean (PREF_FAST_FILE_TYPE, TRUE));
+	priv->mime_type = mime_type;
+
 	if (priv->uri != NULL) {
 		gnome_vfs_uri_unref (priv->uri);
 		priv->uri = NULL;
 	}
-	if (path != NULL) 
+	if (path != NULL)
 		priv->uri = new_uri_from_path (path);
 
 	g_mutex_unlock (priv->yes_or_no);
@@ -357,16 +391,19 @@ image_loader_set_path (ImageLoader *il,
 
 void
 image_loader_set_uri (ImageLoader       *il,
-		      const GnomeVFSURI *uri)
+		      const GnomeVFSURI *uri,
+		      const char        *mime_type)
 {
 	ImageLoaderPrivateData *priv;
 
 	g_return_if_fail (il != NULL);
-	
+
 	priv = il->priv;
 
 	g_mutex_lock (priv->yes_or_no);
 
+	priv->mime_type = mime_type;
+
 	if (priv->uri != NULL) {
 		gnome_vfs_uri_unref (priv->uri);
 		priv->uri = NULL;
@@ -405,29 +442,36 @@ image_loader_set_pixbuf (ImageLoader *il,
 	g_mutex_unlock (priv->yes_or_no);
 }
 
-
-static void 
+static void
 image_loader_sync_pixbuf (ImageLoader *il)
 {
 	GdkPixbuf              *pixbuf;
+	GdkPixbuf              *temp;
 	ImageLoaderPrivateData *priv;
-  
+	ExifShort				orientation;
+	GthTransform			transform;
+
 	g_return_if_fail (il != NULL);
-     
+
 	priv = il->priv;
 
+	orientation = get_exif_tag_short (image_loader_get_path(il), EXIF_TAG_ORIENTATION);
+	transform =  (orientation >= 1 && orientation <= 8 ?
+						orientation :
+						GTH_TRANSFORM_NONE);
+
 	g_mutex_lock (priv->yes_or_no);
 
 	if (priv->animation == NULL) {
-		if (priv->pixbuf != NULL) 
+		if (priv->pixbuf != NULL)
 			g_object_unref (priv->pixbuf);
 		priv->pixbuf = NULL;
 		g_mutex_unlock (priv->yes_or_no);
 		return;
 	}
 
-	pixbuf = gdk_pixbuf_animation_get_static_image (priv->animation);
-	g_object_ref (pixbuf);
+	temp = gdk_pixbuf_animation_get_static_image (priv->animation);
+	pixbuf = _gdk_pixbuf_transform (temp, transform);
 
 	if (priv->pixbuf == pixbuf) {
 		g_object_unref (pixbuf);
@@ -450,15 +494,15 @@ image_loader_sync_pixbuf (ImageLoader *il)
 }
 
 
-static void 
+static void
 image_loader_sync_pixbuf_from_loader (ImageLoader     *il,
 				      GdkPixbufLoader *pb_loader)
 {
 	GdkPixbuf              *pixbuf;
 	ImageLoaderPrivateData *priv;
-  
+
 	g_return_if_fail (il != NULL);
-     
+
 	priv = il->priv;
 
 	g_mutex_lock (priv->yes_or_no);
@@ -501,7 +545,7 @@ image_loader_sync_pixbuf_from_loader (ImageLoader     *il,
 }
 
 
-static void 
+static void
 image_loader_interrupted (ImageLoader *il)
 {
 	ImageLoaderPrivateData *priv = il->priv;
@@ -510,15 +554,15 @@ image_loader_interrupted (ImageLoader *il)
 	priv->error = TRUE;
 	g_mutex_unlock (priv->yes_or_no);
 
-	image_loader_stop_common (il, 
-				  priv->done_func, 
-				  priv->done_func_data, 
+	image_loader_stop_common (il,
+				  priv->done_func,
+				  priv->done_func_data,
 				  TRUE,
 				  TRUE);
 }
 
 
-static void 
+static void
 image_loader_done (ImageLoader *il)
 {
 	ImageLoaderPrivateData *priv = il->priv;
@@ -531,7 +575,7 @@ image_loader_done (ImageLoader *il)
 }
 
 
-static void 
+static void
 image_loader_error (ImageLoader *il)
 {
 	ImageLoaderPrivateData *priv = il->priv;
@@ -577,22 +621,17 @@ load_image_thread (void *thread_data)
 
 		animation = NULL;
 		if (path != NULL) {
-			if (priv->loader != NULL) 
-				animation = (*priv->loader) (path, &error, priv->loader_data);
-			else {
-				if (image_is_gif__accurate (path))
-					animation = gdk_pixbuf_animation_new_from_file (path, &error);
-				else {
-					GdkPixbuf *pixbuf;
-					pixbuf = gdk_pixbuf_new_from_file (path, &error);
-					if (pixbuf != NULL) {
-						animation = gdk_pixbuf_non_anim_new (pixbuf);
-						g_object_unref (pixbuf);
-					}
-				}
-			}
-		} 
-		
+			if (priv->loader != NULL)
+				animation = (*priv->loader) (path, priv->mime_type, &error, priv->loader_data);
+        		else
+		                animation = gth_pixbuf_animation_new_from_uri (path,
+									       &error,
+									       0,
+									       0,
+									       priv->thumb_factory,
+									       priv->mime_type);
+		}
+
 		G_UNLOCK (pixbuf_loader_lock);
 
 		priv->loader_done = TRUE;
@@ -622,7 +661,7 @@ load_image_thread (void *thread_data)
 
 static void
 image_loader_stop__final_step (ImageLoader *il,
-			       gboolean     use_idle_cb) 
+			       gboolean     use_idle_cb)
 {
 	ImageLoaderPrivateData *priv = il->priv;
 	DoneFunc                done_func = priv->done_func;
@@ -633,30 +672,30 @@ image_loader_stop__final_step (ImageLoader *il,
 	priv->done = TRUE;
 	g_mutex_unlock (priv->yes_or_no);
 
-	if (! error && ! priv->interrupted && priv->loading) 
+	if (! error && ! priv->interrupted && priv->loading)
 		image_loader_sync_pixbuf (il);
 	priv->loading = FALSE;
-	
+
 	priv->done_func = NULL;
 	if (done_func != NULL) {
 		IdleCall* call = idle_call_new (done_func, priv->done_func_data);
-		if (priv->idle_id != 0) 
+		if (priv->idle_id != 0)
 			g_source_remove (priv->idle_id);
 		priv->idle_id = idle_call_exec (call, use_idle_cb);
 	}
-	
+
 	if (! priv->emit_signal || priv->interrupted) {
 		priv->interrupted = FALSE;
 		return;
 	}
-		
+
 	priv->interrupted = FALSE;
 
 	if (error)
-		g_signal_emit (G_OBJECT (il), 
+		g_signal_emit (G_OBJECT (il),
 			       image_loader_signals[IMAGE_ERROR], 0);
 	else
-		g_signal_emit (G_OBJECT (il), 
+		g_signal_emit (G_OBJECT (il),
 			       image_loader_signals[IMAGE_DONE], 0);
 }
 
@@ -670,7 +709,7 @@ check_thread (gpointer data)
 
 	/* Remove check. */
 
-	g_source_remove (priv->check_id);	
+	g_source_remove (priv->check_id);
 	priv->check_id = 0;
 
 	/**/
@@ -686,15 +725,15 @@ check_thread (gpointer data)
 	if (loader_done && priv->interrupted)
 		image_loader_interrupted (il);
 
-	else if (loader_done && done) 
+	else if (loader_done && done)
 		image_loader_done (il);
-	
-	else if (loader_done && error) 
+
+	else if (loader_done && error)
 		image_loader_error (il);
-	
+
 	else	/* Add check again. */
 		priv->check_id = g_timeout_add (REFRESH_RATE, check_thread, il);
-	
+
 	return FALSE;
 }
 
@@ -735,7 +774,7 @@ get_file_info_cb (GnomeVFSAsyncHandle *handle,
 	priv->start_loading = TRUE;
 	g_cond_signal (priv->start_loading_cond);
 	g_mutex_unlock (priv->start_loading_mutex);
-	
+
 	il->priv->check_id = g_timeout_add (REFRESH_RATE, check_thread, il);
 }
 
@@ -774,7 +813,7 @@ image_loader_start__step2 (ImageLoader *il)
 
 	{
 		GList *scan;
-		for (scan = uri_list; scan; scan = scan->next) 
+		for (scan = uri_list; scan; scan = scan->next)
 			gnome_vfs_uri_unref((GnomeVFSURI*) scan->data);
 		g_list_free (uri_list);
 	}
@@ -792,14 +831,14 @@ image_loader_start (ImageLoader *il)
 
 	g_mutex_lock (priv->yes_or_no);
 	if (priv->uri == NULL) {
-		g_mutex_unlock (priv->yes_or_no);		
+		g_mutex_unlock (priv->yes_or_no);
 		return;
 	}
 	g_mutex_unlock (priv->yes_or_no);
 
-	image_loader_stop_common (il, 
-				  (DoneFunc) image_loader_start__step2, 
-				  il, 
+	image_loader_stop_common (il,
+				  (DoneFunc) image_loader_start__step2,
+				  il,
 				  FALSE,
 				  TRUE);
 }
@@ -843,7 +882,7 @@ image_loader_stop_common (ImageLoader *il,
 	priv->done_func_data = done_func_data;
 	priv->emit_signal = emit_sig;
 
-	if (priv->info_handle != NULL) 
+	if (priv->info_handle != NULL)
 		gnome_vfs_async_close (priv->info_handle, close_info_cb, il);
 
 	priv->info_handle = NULL;
@@ -851,7 +890,7 @@ image_loader_stop_common (ImageLoader *il,
 }
 
 
-void 
+void
 image_loader_stop (ImageLoader *il,
 		   DoneFunc     done_func,
 		   gpointer     done_func_data)
@@ -872,15 +911,15 @@ image_loader_stop (ImageLoader *il,
 		priv->done_func = done_func;
 		priv->done_func_data = done_func_data;
 	} else
-		image_loader_stop_common (il, 
-					  done_func, 
-					  done_func_data, 
-					  FALSE, 
+		image_loader_stop_common (il,
+					  done_func,
+					  done_func_data,
+					  FALSE,
 					  TRUE);
 }
 
 
-void 
+void
 image_loader_stop_with_error (ImageLoader *il,
 			      DoneFunc     done_func,
 			      gpointer     done_func_data)
@@ -938,7 +977,7 @@ image_loader_get_animation (ImageLoader *il)
 }
 
 
-float 
+float
 image_loader_get_percent (ImageLoader *il)
 {
 	ImageLoaderPrivateData *priv;
@@ -946,14 +985,14 @@ image_loader_get_percent (ImageLoader *il)
 	g_return_val_if_fail (il != NULL, 0.0);
 	priv = il->priv;
 
-	if (priv->bytes_total == 0) 
+	if (priv->bytes_total == 0)
 		return 0.0;
 	else
 		return (float) priv->bytes_read / priv->bytes_total;
 }
 
 
-gint 
+gint
 image_loader_get_is_done (ImageLoader *il)
 {
 	ImageLoaderPrivateData *priv;
@@ -974,9 +1013,8 @@ gchar *
 image_loader_get_path (ImageLoader *il)
 {
 	ImageLoaderPrivateData *priv;
-	char                   *path;
-        char                   *esc_path;
 	GnomeVFSURI            *uri;
+	char                   *path;
 
 	g_return_val_if_fail (il != NULL, 0);
 	priv = il->priv;
@@ -989,15 +1027,12 @@ image_loader_get_path (ImageLoader *il)
 	}
 
 	uri = gnome_vfs_uri_dup (priv->uri);
-
-        path = gnome_vfs_uri_to_string (uri, GNOME_VFS_URI_HIDE_TOPLEVEL_METHOD);
-        esc_path = gnome_vfs_unescape_string (path, NULL);
-        g_free (path);
+        path = gnome_vfs_uri_to_string (uri, GNOME_VFS_URI_HIDE_NONE);
 	gnome_vfs_uri_unref (uri);
 
 	g_mutex_unlock (priv->yes_or_no);
 
-        return esc_path;
+        return path;
 }
 
 
@@ -1034,7 +1069,7 @@ image_loader_load_from_pixbuf_loader (ImageLoader *il,
 	gboolean error;
 
 	g_return_if_fail (il != NULL);
-	
+
 	image_loader_sync_pixbuf_from_loader (il, pixbuf_loader);
 
 	g_mutex_lock (il->priv->yes_or_no);
